package com.link.core.basic.es.parse;

import com.link.core.basic.es.util.EsCacheUtils;
import com.link.core.basic.es.util.EsFieldType;
import com.link.core.basic.es.util.EsUtil;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.util.StringUtils;
import java.util.Map;
import javax.annotation.Resource;
import org.elasticsearch.script.Script;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.collapse.CollapseBuilder;
import org.springframework.stereotype.Component;

@Component
public class EsDistinctParse {
   @Resource
   private EsCacheUtils esCacheUtils;
   public static final String NESTED_DISTINCT_RESULT = "distinctNestedField_";

   public void parseFields(SearchSourceBuilder sourceBuilder, BasicModel basicModel) throws Exception {
      if (StringUtils.isBlank(basicModel.getEsIndex())) {
         throw new BasicServiceException("Es目标索引名称为空");
      } else {
         String[] distinctFields = basicModel.getDistinctFields().split(",");
         if (distinctFields.length > 0) {
            String fieldType;
            if (distinctFields.length == 1) {
               String orgField = distinctFields[0];
               Map<String, String> map = EsUtil.parseNestedField(orgField, basicModel);
               String field = (String)map.get(EsFieldType.NESTED_OBJECT.getValue());
               String[] fields = field.split("\\.");
               fieldType = (String)map.get(EsFieldType.RUNTIME.getValue());
               if (StringUtils.isBlank(fieldType)) {
                  fieldType = this.esCacheUtils.getFieldType(basicModel.getEsIndex(), field.split("\\."));
               }

               if (!EsFieldType.KEYWORD.getValue().equals(fieldType)) {
                  field = field + ".keyword";
               }

               if (fields.length == 1) {
                  CollapseBuilder collapseBuilder = new CollapseBuilder(field);
                  sourceBuilder.collapse(collapseBuilder);
               }
            } else {
               StringBuilder esScript = new StringBuilder();
               String separator = "+'-'+";

               for(int i = 0; i < distinctFields.length; ++i) {
                  String item = distinctFields[i];
                  fieldType = this.esCacheUtils.getFieldType(basicModel.getEsIndex(), new String[]{item});
                  if (!EsFieldType.KEYWORD.getValue().equals(fieldType)) {
                     item = item + ".keyword";
                  }

                  if (i == distinctFields.length - 1) {
                     separator = "";
                  }

                  esScript.append("doc['").append(item).append("']").append(separator);
               }

               Script script = new Script(esScript.toString());
               AggregationBuilder termsBuilder = AggregationBuilders.terms("result").script(script);
               sourceBuilder.aggregation(termsBuilder);
               sourceBuilder.size(0);
            }

         }
      }
   }
}
